@@ -4,29 +4,31 @@ import confuse
 
 __author__ = "Alex Laird"
 __copyright__ = "Copyright 2020, Alex Laird"
-__version__ = "0.0.12"
+__version__ = "1.0.1"
 
 template = {
-    "auth_token": confuse.String(default=None),
-    "region": confuse.Choice(["us", "eu", "ap", "au", "sa", "jp", "in"], default=None),
-    "subdomain": confuse.String(default=None),
-    "auth": confuse.String(default=None),
     "port": int,
+    "subdomain": confuse.String(default=None),
+    "region": confuse.Choice(["us", "eu", "ap", "au", "sa", "jp", "in"], default=None),
+    "hostname": confuse.String(default=None),
+    "auth": confuse.String(default=None),
+    "host_header": confuse.String(default=None),
+    "response": confuse.String(default=None),
+    "content_type": confuse.String(default=None),
+    "request_script": confuse.Filename(default=None),
+    "response_script": confuse.Filename(default=None),
+    "auth_token": confuse.String(default=None),
     "plugins_dir": confuse.Filename(),
     "plugins": list,
     "console_width": confuse.Integer(default=80),
-    "request_script": confuse.Filename(default=None),
-    "response_script": confuse.Filename(default=None),
-    "response": confuse.String(default=None),
-    "content_type": confuse.String(default=None)
 }
 
 
 class Config:
     """
     An object with accessor methods containing ``hookee``'s configuration. Default configuration can be
-    overridden by creating a custom ``config.yaml`` in ``~/.config/hookee`` (when setting config
-    values from the command lind, this is where values are updated) which in turn can be overridden by
+    overridden by creating a custom config at ``~/.config/hookee/config.yaml`` (when setting config
+    values from the command line, this is where updated values are stored) which in turn can be overridden by
     passing args to the CLI.
 
     :var ctx: The ``click`` CLI context.
@@ -76,7 +78,7 @@ class Config:
 
     def set(self, key, value):
         """
-        Update the config value for the given key, persisting to user's ``config.yaml``.
+        Update the config key to the given value, persisting to ``config.yaml``.
 
         :param key: The key.
         :type key: str
@@ -88,8 +90,7 @@ class Config:
 
     def append(self, key, value):
         """
-        Update the config value by appending to the list for the given key, persisting to
-        user's ``config.yaml``.
+        Update the config key by appending to the list the given value, persisting to ``config.yaml``.
 
         :param key: The key.
         :type key: str
@@ -104,8 +105,8 @@ class Config:
 
     def remove(self, key, value):
         """
-        Update the config value by removing the given value from the list for the given key, persisting to
-        user's ``config.yaml``.
+        Update the config key by removing from the list the given value from the list for the given key, persisting to
+        ``config.yaml``.
 
         :param key: The key.
         :type key: str
