@@ -1,26 +1,25 @@
 import click
 
-from hookee import util
+from hookee import pluginmanager
 
 __author__ = "Alex Laird"
 __copyright__ = "Copyright 2020, Alex Laird"
-__version__ = "0.0.4"
+__version__ = "0.0.7"
 
-plugin_type = util.REQUEST_PLUGIN
-manager = None
+plugin_type = pluginmanager.REQUEST_PLUGIN
+print_util = None
 
 
-def setup(_manager):
-    global manager
+def setup(cli_manager):
+    global print_util
 
-    manager = _manager
+    print_util = cli_manager.print_util
 
 
 def run(request):
-    # TODO: pretty this up further
     if request.headers and "X-Forwarded-For" in request.headers:
         click.secho("Client IP: {}".format(request.headers.get("X-Forwarded-For")), fg="magenta")
     if request.headers:
-        click.secho("Headers: {}".format(dict(request.headers)), fg="magenta")
+        print_util.print_dict("Headers", dict(request.headers), fg="magenta")
 
     return request
